;  ALIEN-GLOW
;  Create a text effect that simulates an eerie alien glow around text

; GIMP - The GNU Image Manipulation Program
; Copyright (C) 1995 Spencer Kimball and Peter Mattis
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.
; ------------
;| Change Log |
; ------------
; Rel 1: Initial release.
; Rel 2: MrQ: procedures adapted to Gimp-2.10, added adjust and text direction,  for Alpha To Logo- creates a new image from the selected layer.
; Rel 3: MrQ: add border, type background.

; start add border----------------------------------------------------------------------
(define (resize-image-to-layers-by-border inImg inDrawable inBufferSize)
    (let* (
        (theImgWidth (car(gimp-image-width inImg)))
        (theImgHeight (car(gimp-image-height inImg)))
        (theIterations 0)
        (theLayerNum (car(gimp-image-get-layers inImg)))
        (theWorkingLayer 0)
        (theLayerWidth 0)
        (theLayerHeight 0)
        (theLayerOffsetX 0)
        (theLayerOffsetY 0)
        (theBorder FALSE)
        (theMinPosX theImgWidth)
        (theMaxPosX 0)
        (theMinPosY theImgHeight)
        (theMaxPosY 0)
        (theNewImgWidth theImgWidth)
        (theNewImgHeight theImgHeight)
        )

    (gimp-image-undo-group-start inImg)

    (begin
     (while (< theIterations theLayerNum)
        (set! theWorkingLayer (aref (cadr (gimp-image-get-layers inImg)) theIterations))
        (gimp-image-set-active-layer inImg theWorkingLayer)

        (set! theLayerOffsetX (car(gimp-drawable-offsets theWorkingLayer)))
        (set! theLayerOffsetY (cadr(gimp-drawable-offsets theWorkingLayer)))
        (set! theLayerWidth (car(gimp-drawable-width theWorkingLayer)))
        (set! theLayerHeight (car(gimp-drawable-height theWorkingLayer)))

        (cond ((> (+ theLayerWidth theLayerOffsetX) theImgWidth) (set! theBorder TRUE))
              ((> (+ theLayerWidth theLayerOffsetX) theMaxPosX) (set! theMaxPosX (+ theLayerWidth theLayerOffsetX)))
        );end cond

        (cond ((> (+ theLayerHeight theLayerOffsetY) theImgHeight) (set! theBorder TRUE))
              ((> (+ theLayerHeight theLayerOffsetY) theMaxPosY) (set! theMaxPosY (+ theLayerHeight theLayerOffsetY)))
        );end cond

        (cond ((< theLayerOffsetX 0) (set! theBorder TRUE))
              ((< theLayerOffsetX theMinPosX) (set! theMinPosX theLayerOffsetX))
        );end cond

        (cond ((< theLayerOffsetY 0) (set! theBorder TRUE))
              ((< theLayerOffsetY theMinPosY) (set! theMinPosY theLayerOffsetY))
        );end cond

        (set! theIterations (+ theIterations 1))
     );end while
    );end begin

    (if (equal? theBorder TRUE) (set! theMinPosX 0) (set! theMaxPosX (- theMaxPosX theMinPosX)))
    (if (equal? theBorder TRUE) (set! theMinPosY 0) (set! theMaxPosY (- theMaxPosY theMinPosY)))
    (if (equal? theBorder TRUE) (set! theMaxPosX (- theImgWidth theMinPosX)))
    (if (equal? theBorder TRUE) (set! theMaxPosY (- theImgHeight theMinPosY)))

    ;resize the image
    (gimp-image-resize inImg theMaxPosX theMaxPosY (- theMinPosX) (- theMinPosY))

    ;add the border. Default is 0, only useful if set by the user
    (set! theImgWidth (+ (car(gimp-image-width inImg)) inBufferSize inBufferSize)) ;szerokość
    (set! theImgHeight (+ (car(gimp-image-height inImg)) inBufferSize inBufferSize))			;wysokość

    ;resize the image to add the border
    (gimp-image-resize inImg theImgWidth theImgHeight inBufferSize inBufferSize)

    (gimp-selection-none inImg)
	
	(map (lambda (x) (gimp-layer-resize-to-image-size x)) (vector->list (cadr (gimp-image-get-layers inImg))))
    ;(begin (gimp-layer-resize-to-image-size inDrawable))
    (gimp-image-undo-group-end inImg)
    (gimp-displays-flush)
    );end let
);end define

; end add border----------------------------------------------------------------------

(define (apply-alien-glow-logo-effect img
                                      logo-layer
                                      size
									  border
                                      glow-color
									  bkg-type
									  color
									  pattern
									  gradient)
  (let* (
        ;(border 0)
        (grow (/ size 30))
        (feather (/ size 4))
        (width (car (gimp-drawable-width logo-layer)))
        (height (car (gimp-drawable-height logo-layer)))
			(gimp-layer-set-offsets logo-layer 0 border)
        (bkg-layer (car (gimp-layer-new img
                                       width height RGB-IMAGE
                                       "Background" 100 LAYER-MODE-NORMAL-LEGACY)))
        (glow-layer (car (gimp-layer-new img
                                         width height RGBA-IMAGE
                                         "Alien Glow" 100 LAYER-MODE-NORMAL-LEGACY)))
        )

    (gimp-context-push)
    (gimp-context-set-defaults)

    (gimp-selection-none img)
    (script-fu-util-image-resize-from-layer img logo-layer)
    (script-fu-util-image-add-layers img glow-layer bkg-layer)
	    (resize-image-to-layers-by-border img logo-layer border)
    (gimp-layer-set-lock-alpha logo-layer TRUE)
	
	(if (= bkg-type 0);; color
		(begin
    (gimp-context-set-background color)
    (gimp-drawable-fill bkg-layer FILL-BACKGROUND)
           )
    )	
    
    (if (= bkg-type 1);Pattern
		(begin
    (gimp-context-set-pattern pattern);;test
    (gimp-drawable-fill bkg-layer FILL-PATTERN)
           )
    )
	
    (if (= bkg-type 2);;Gradient
		(begin
    (gimp-context-set-background '(255 255 255))
    (gimp-drawable-fill bkg-layer FILL-BACKGROUND)
    (gimp-context-set-gradient gradient)    
    (gimp-edit-blend bkg-layer CUSTOM-MODE  LAYER-MODE-NORMAL-LEGACY GRADIENT-LINEAR 100 0 REPEAT-NONE FALSE FALSE 3 0.2 TRUE 0 0 width height);; GRADIENT-LINEAR
           )
    )
	
    (gimp-edit-clear glow-layer)
    (gimp-image-select-item img CHANNEL-OP-REPLACE logo-layer)
    (gimp-selection-grow img grow)
    (gimp-selection-feather img feather)
    (gimp-context-set-foreground glow-color)
    (gimp-edit-fill glow-layer FILL-FOREGROUND)
    (gimp-selection-none img)

    (gimp-context-set-background '(0 0 0))
    (gimp-context-set-foreground '(79 79 79))

    (gimp-edit-blend logo-layer BLEND-FG-BG-RGB LAYER-MODE-NORMAL-LEGACY
                     GRADIENT-SHAPEBURST-ANGULAR 100 0 REPEAT-NONE FALSE
                     FALSE 0 0 TRUE
                     0 0 1 1)

    (gimp-context-pop)
  )
)

(define (script-fu-alien-glow-logo-alpha image
                                         layer
										 create-new
										 border
                                         size
                                         glow-color
										 bkg-type
										 color
										 pattern
										 gradient)
										 
    (if (= create-new TRUE)
	    (begin(gimp-image-undo-group-start image)
		
; star layer to new image - add MrQ ----------------------------------------------------------
			(let*	(
				(active_layer (car (gimp-image-get-active-layer image)))
				(image-type (car (gimp-image-base-type image)))
				(width (car (gimp-drawable-width active_layer)))
				(height (car (gimp-drawable-height active_layer)))
				(new-image 0)
				(new-layer 0)
				(aa 0)
					)
				(gimp-image-undo-disable image)
				(set! new-image (car (gimp-image-new width height image-type)))
				(gimp-display-new new-image)
				(set! new-layer (car (gimp-layer-new-from-drawable layer new-image)))
				(set! aa (gimp-image-insert-layer new-image new-layer 0 0))
				(gimp-image-undo-enable image)

; end new image MrQ -----------------------------------------------------------------------------------------------------

				(apply-alien-glow-logo-effect new-image new-layer size border glow-color bkg-type color pattern gradient)
				(gimp-image-undo-group-end image)
				(gimp-displays-flush)
	        )	
        )
	)	
	(if (= create-new FALSE)
		(begin
		    (gimp-image-undo-group-start image)
			(apply-alien-glow-logo-effect image layer size glow-colorbkg-type color pattern gradient)
			(gimp-image-undo-group-end image)
			(gimp-displays-flush)
		)
    )
)
(script-fu-register "script-fu-alien-glow-logo-alpha"
  _"Alien _Glow..."
  _"Add an eerie glow around the selected region (or alpha)"
  "Spencer Kimball/MrQ"
  "GPL"
  "1997/2020"
  "RGBA"
  SF-IMAGE       "Image"                  0
  SF-DRAWABLE    "Drawable"               0
  SF-TOGGLE     _"Layer to New Image"   TRUE
  SF-ADJUSTMENT _"Add Border" '(10 0 1000 1 5 0 0)
  SF-ADJUSTMENT _"Glow size (pixels * 4)" '(150 2 1000 1 10 0 1)
  SF-COLOR      _"Glow color"             '(63 252 0)
  SF-OPTION     _"Texture Type" '("Color" "Pattern" "Gradient")
  SF-COLOR      _"Texture Color"    '(0 0 0)
  SF-PATTERN    _"Texture Pattern"  "Blue Web"
  SF-GRADIENT   _"Texture Gradient" "Incandescent"
)

(script-fu-menu-register "script-fu-alien-glow-logo-alpha"
                         "<Image>/Filters/Alpha to Logo")


(define (script-fu-alien-glow-logo text border direction justify letter-spacing line-spacing size font glow-color bkg-type color pattern gradient)
  (let* (
        (img (car (gimp-image-new 256 256 RGB)))
        (grow (/ size 30))
        (feather (/ size 4))
        (text-layer (car (gimp-text-fontname img -1 0 0 text 0 TRUE size PIXELS font)))
											 
        (width (car (gimp-drawable-width text-layer)))
        (height (car (gimp-drawable-height text-layer)))
		
		(justify (cond ((= justify 0) 2)
			           ((= justify 1) 0)
			           ((= justify 2) 1)
			           ((= justify 3) 3)))
        
		)
	(gimp-text-layer-set-justification text-layer justify)
	(gimp-text-layer-set-letter-spacing text-layer letter-spacing)
	(gimp-text-layer-set-line-spacing text-layer line-spacing)		
	(gimp-text-layer-set-base-direction text-layer direction)

	(gimp-image-undo-disable img)
	(apply-alien-glow-logo-effect img text-layer size border glow-color bkg-type color pattern gradient)
	(gimp-image-undo-enable img)
	(gimp-display-new img)
  )
)

(script-fu-register "script-fu-alien-glow-logo"
  _"Alien _Glow..."
  _"Create a logo with an alien glow around the text"
  "Spencer Kimball/MrQ"
  "GPL"
  "1997/2020"
  ""
	SF-TEXT _"Text" "Gimp 2.10.20\nCreate Logo"
	SF-ADJUSTMENT _"Add Border" '(20 0 1000 1 5 0 0) 
	SF-OPTION     _"Direction" '("From left to right"  "From right to left" "Vertical, right to left (mixed orientation)" "Vertical, right to left (upright orientation)" "Vertical, left to right (mixed orientation)" "Vertical, left to right (upright orientation)")
	;SF-OPTION     _"Direction" '("Od lewej do prawej"  "Od prawej do lewej" "Pionowy, od prawej do lewej (mieszana orientacja)" "Pionowy, od prawej do lewej (pionowa orientacja)" "Pionowy, od lewej do prawej (mieszana orientacja)" "Pionowy, od lewej do prawej (pionowa orientacja)")	
	SF-OPTION     _"Justify" '("Centered" "Left" "Right" "Fill")   ;add MrQ
	SF-ADJUSTMENT _"Letter Spacing" '(0 -100 100 1 5 0 0)          ;add MrQ
	SF-ADJUSTMENT _"Line Spacing" '(0 -100 100 1 5 0 0)            ;add MrQ	
	SF-ADJUSTMENT _"Font size (pixels)" '(150 2 1000 1 10 0 1)
	SF-FONT       _"Font"               "Sans Bold"
	SF-COLOR      _"Glow color"         '(63 252 0)
    SF-OPTION     _"Texture Type" '("Color" "Pattern" "Gradient") ;add MrQ	
    SF-COLOR      _"Texture Color"    '(0 0 0)                    ;add MrQ	
    SF-PATTERN    _"Texture Pattern"  "Blue Web"                  ;add MrQ	
    SF-GRADIENT   _"Texture Gradient" "Incandescent"	          ;add MrQ		
)

(script-fu-menu-register "script-fu-alien-glow-logo"
                         "<Image>/File/Create/Logos")
